==> thzprop-1.0.tar.gz: thzprop-1.0/complexfiles.cpp <==
// Will do nothing if __MYDEBUG is not defined
#include "mydebug.h"
#include <iostream>
#include <complex>
#include <valarray>
#include "fourier.h"
#include "optics.h"
#include "complexfiles.h"

using namespace std;

bool MakeHermitian( const std::valarray<double>& x, std::valarray<Complex>& y)
{
	for (unsigned long i = 0; i < x.size(); ++i) {
		if (x[i] < 0 ) {
			unsigned long j;
			for (j = 0; j < x.size(); ++j) {
				if (x[j] == - x[i])
					break;	
			}
			
			y[i] = conj(y[j]);
		}
	}
	
	return true;
}

int
WriteComplexFunctionsToDisk(	valarray<double>& inX,
								valarray<complex_double>& inY,
								string inAbsfile,
								string inArgfile)
{
    ofstream fAbs(inAbsfile.c_str());
    ofstream fArg(inArgfile.c_str());

	if (fAbs.bad()) {
		cerr << "Problems opening file: " << inAbsfile << endl;
		throw;
	}

	if (fArg.bad()) {
		cerr << "Problems opening file: " << inArgfile << endl;
		throw;
	}

	if (CheckRealArray_(inX)) {
		cout << inAbsfile << " " << inArgfile << endl;
	}
	
	if (CheckArray_(inY)) {
		cout << inAbsfile << " " << inArgfile << endl;
	}


	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same size" << endl;
		throw;
	}
		
	unsigned long N = inX.size();
	
	for (unsigned long i = 0; i < N; ++i) {
		fAbs << inX[i] << "\t" << abs(inY[i]) << endl;
		fArg << inX[i] << "\t" << arg(inY[i]) << endl;

		if (fAbs.bad() || fArg.bad()) {
			cerr << "Problems writing to files" << endl;
			throw;
		}
	}

	return true;

}

int
ReadComplexFunctionsFromDisk(	valarray<double>& inX,
								valarray<complex_double>& inY,
								string inAbsfile,
								string inArgfile,
								int inFrequencyCheck)
{
    ifstream fAbs(inAbsfile.c_str());
    ifstream fArg(inArgfile.c_str());

	if (fAbs.bad()) {
		cerr << "Problems opening file: " << inAbsfile << endl;
		throw;
	}

	if (fArg.bad()) {
		cerr << "Problems opening file: " << inArgfile << endl;
		throw;
	}

		
	valarray<double> tx(32768);
	valarray<double> tabs(32768);
	valarray<double> targ(32768);

	unsigned long N = 0;
		
	while (!fAbs.eof()) {
		double x, x1, x2, absvalue,argvalue;
		fAbs >> x1 >> absvalue;
		fArg >> x2 >> argvalue;

		if (x1 != x2) {
			cerr << "X values not identical" << endl;
			throw;
		} else {
			x = x1;
		}

		if (!fAbs.good() || !fArg.good())
			break;

		tx[N] = x;
		tabs[N] = absvalue;
		targ[N] = argvalue;
		
		++N;

		if (N >= 32768) {
			cerr << "Too many points (for an eventual FFT).\n";
			exit(1);
		}

	}

	// N is number of points [0..N-1]
	// We can't resize without reinitializing

	inX.resize(N);
	inY.resize(N);
	
	for (unsigned long i = 0; i < N; ++i) {
		inX[i] = tx[i];
		inY[i] = tabs[i]*exp(I*targ[i]);
	}


	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same same" << endl;
		throw;
	}

	// Finally, we organize frequencies like the user wants them
	// We have to swap point i with i + N/2

	if (inFrequencyCheck == kMakeSureXIsValidForFFT) {
		if (inX[0] < 0) {
			for (unsigned long i = 0; i < inY.size() / 2; ++i) {
				swap<double>(inX[i],inX[i + inY.size()/2]);
				swap<complex_double>(inY[i],inY[i + inY.size()/2]);
			}
		}
		CheckIfValidXForFFT_(inX);
	} 
	
	CheckRealArray_(inX);
	CheckArray_(inY);

	return true;

}

int
WriteRealFFTToDisk(	valarray<double>& inX,
					valarray<complex_double>& inY,
					short direction,
					string inFile)
{
    ofstream file(inFile.c_str());
	if (file.bad()) {
		cerr << "Problems opening file: " << inFile << endl;
		throw;
	}

	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same size" << endl;
		throw;
	}

	CheckRealArray_(inX);
	CheckIfHermitian_(inX, inY);
	
	valarray<complex_double> c = inY;

	long N = long(inX.size());
	double dX,Xmax,dF,Fmax;
	ExtractFFTParams(inX, dX,Xmax, dF,Fmax);

	FFT(c, direction);
	
	// Normalize 
	c *= dX;

	// We turn it around...
	c = c.cshift( int(N/2) );
	
	for ( long i = 0; i < N; ++i) {
		double F;

		// ... and do the same for the X axis
		F = ( i - N/2 ) * dF; 		

/*
		if (i <= N/2-1) {
			F =  i * dF;
		} else {
			F = (i - N) * dF;
		}
*/
		
		file <<  F << "\t" << real(c[i]) << endl;

		if (file.bad()) {
			cerr << "Problems writing to files" << endl;
			throw;
		}
	}
	
	return true;


}

==> thzprop-1.1.tar.gz: thzprop-1.1/complexfiles.cpp <==
// Will do nothing if __MYDEBUG is not defined
#include "mydebug.h"
#include <iostream>
#include <complex>
#include <valarray>
#include "fourier.h"
#include "optics.h"
#include "complexfiles.h"

using namespace std;

bool MakeHermitian( const std::valarray<double>& x, std::valarray<Complex>& y)
{
	for (unsigned long i = 0; i < x.size(); ++i) {
		if (x[i] < 0 ) {
			unsigned long j;
			for (j = 0; j < x.size(); ++j) {
				if (x[j] == - x[i])
					break;	
			}
			
			y[i] = conj(y[j]);
		}
	}
	
	return true;
}

int
WriteComplexFunctionsToDisk(	valarray<double>& inX,
                                valarray<complex_double>& inY,
                                string inAbsfile,
                                string inArgfile)
{
    ofstream fAbs(inAbsfile.c_str());
    ofstream fArg(inArgfile.c_str());

	if (fAbs.bad()) {
		cerr << "Problems opening file: " << inAbsfile << endl;
		throw;
	}

	if (fArg.bad()) {
		cerr << "Problems opening file: " << inArgfile << endl;
		throw;
	}

	if (CheckRealArray_(inX)) {
		cerr << inAbsfile << " " << inArgfile << endl;
	}
	
	if (CheckArray_(inY)) {
		cerr << inAbsfile << " " << inArgfile << endl;
	}


	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same size" << endl;
		throw;
	}
		
	unsigned long N = inX.size();
	
	for (unsigned long i = 0; i < N; ++i) {
		fAbs << inX[i] << "\t" << abs(inY[i]) << endl;
		fArg << inX[i] << "\t" << arg(inY[i]) << endl;

		if (fAbs.bad() || fArg.bad()) {
			cerr << "Problems writing to files" << endl;
			throw;
		}
	}

	return true;

}

int
ReadComplexFunctionsFromDisk(	valarray<double>& inX,
								valarray<complex_double>& inY,
								string inAbsfile,
								string inArgfile,
								int inFrequencyCheck)
{
    ifstream fAbs(inAbsfile.c_str());
    ifstream fArg(inArgfile.c_str());

	if (fAbs.bad()) {
		cerr << "Problems opening file: " << inAbsfile << endl;
		throw;
	}

	if (fArg.bad()) {
		cerr << "Problems opening file: " << inArgfile << endl;
		throw;
	}

		
	valarray<double> tx(32768);
	valarray<double> tabs(32768);
	valarray<double> targ(32768);

	unsigned long N = 0;
		
	while (!fAbs.eof()) {
		double x, x1, x2, absvalue,argvalue;
		fAbs >> x1 >> absvalue;
		fArg >> x2 >> argvalue;

		if (x1 != x2) {
			cerr << "X values not identical" << endl;
			throw;
		} else {
			x = x1;
		}

		if (!fAbs.good() || !fArg.good())
			break;

		tx[N] = x;
		tabs[N] = absvalue;
		targ[N] = argvalue;
		
		++N;

		if (N >= 32768) {
			cerr << "Too many points (for an eventual FFT).\n";
			exit(1);
		}

	}

	// N is number of points [0..N-1]
	// We can't resize without reinitializing

	inX.resize(N);
	inY.resize(N);
	
	for (unsigned long i = 0; i < N; ++i) {
		inX[i] = tx[i];
		inY[i] = tabs[i]*exp(I*targ[i]);
	}


	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same same" << endl;
		throw;
	}

	// Finally, we organize frequencies like the user wants them
	// We have to swap point i with i + N/2

	if (inFrequencyCheck == kMakeSureXIsValidForFFT) {
		if (inX[0] < 0) {
			for (unsigned long i = 0; i < inY.size() / 2; ++i) {
				swap<double>(inX[i],inX[i + inY.size()/2]);
				swap<complex_double>(inY[i],inY[i + inY.size()/2]);
			}
		}
		CheckIfValidXForFFT_(inX);
	} 
	
	CheckRealArray_(inX);
	CheckArray_(inY);

	return true;

}

int
WriteRealFFTToDisk(	valarray<double>& inX,
					valarray<complex_double>& inY,
					short direction,
					string inFile)
{
    ofstream file(inFile.c_str());
	if (file.bad()) {
		cerr << "Problems opening file: " << inFile << endl;
		throw;
	}

	if (inX.size() != inY.size()) {
		cerr << "X and Y are not the same size" << endl;
		throw;
	}

	CheckRealArray_(inX);
	CheckIfHermitian_(inX, inY);
	
	valarray<complex_double> c = inY;

	long N = long(inX.size());
	double dX,Xmax,dF,Fmax;
	ExtractFFTParams(inX, dX,Xmax, dF,Fmax);

	FFT(c, direction);
	
	// Normalize 
	c *= dX;

	// We turn it around...
	c = c.cshift( int(N/2) );
	
	for ( long i = 0; i < N; ++i) {
		double F;

		// ... and do the same for the X axis
		F = ( i - N/2 ) * dF; 		

/*
		if (i <= N/2-1) {
			F =  i * dF;
		} else {
			F = (i - N) * dF;
		}
*/
		
		file <<  F << "\t" << real(c[i]) << endl;

		if (file.bad()) {
			cerr << "Problems writing to files" << endl;
			throw;
		}
	}
	
	return true;


}
